from bokeh.server.server import Server
from bokeh.command.util import build_single_handler_applications
import tornado.wsgi
import tornado.httpserver
import tornado.ioloop
import tornado.options
import tornado.autoreload
import tornado.gen
import sys
import threading
import time
import pandas as pd
import os
from functools import partial

class LevServer(object):
	def __init__(self):

		#initialization
		self.app_names = ['bokeh_doc.py']
		self.app_path_dict = {'bokeh_doc.py':None}


	def setup_bokeh(self):
		#turn file paths into bokeh apps
		apps = build_single_handler_applications(self.app_path_dict.keys(),self.app_path_dict)
		
		#kwargs lifted from bokeh serve call to Server, with the addition of my own io_loop
		kwargs = {
			'io_loop':self.io_loop,
			'generade_session_ids':True,
			'redirect_root':True,
			'use_x_headers':False,
			'secret_key':None,
			'num_procs':1,
			'host':['%s:%d'%(self.host,self.app_port),'%s:%d'%(self.host,self.bokeh_port)],
			'sign_sessions':False,
			'develop':False,
			'port':self.bokeh_port,
			'use_index':True
		}
		#instantiate the bokeh server
		srv = Server(apps,**kwargs)

		#return bokeh server object
		return srv

	@tornado.gen.coroutine
	def send_data_to_document(self,item_to_change,data):
		item_to_change.data = data	



	def update_data(self):
		old_time = os.path.getmtime('data.csv')
		while True:
			#polling to see if the csv has been updated
			if os.path.getmtime('data.csv') != old_time:
				old_time = os.path.getmtime('data.csv')	
				print('thar be changes')

				#loops through the applications (of which there are 1), and the living sessions within them
				for app_name,app_context in self.bokeh_server._tornado._applications.items():
					for k,ses in app_context._sessions.items():
						#read the new csv data
						new_data = pd.read_csv('data.csv')['data'].tolist()
						#schedultes a next tick callback to update the data
						ses._document.add_next_tick_callback(
							partial(
								self.send_data_to_document,#the method
								item_to_change = ses._document.select_one({'name':'source'}), #selects the CDS named source
								data = {'dat':new_data} #sends the data in a nice format
							)
						)

				
			time.sleep(1)


	def start_server(self,host='localhost', app_port=9876,bok_port=5006):
		#initialize the server settings
		self.bokeh_port = bok_port
		self.app_port=app_port
		self.host = host
		self.io_loop = tornado.ioloop.IOLoop.instance()

		#sets the bokeh io_loop to be my io_loop
		self.bokeh_server = self.setup_bokeh()

		#start the thread for starting data
		ud_thread = threading.Thread(target=self.update_data)
		ud_thread.start()

		print('starting server on %s:%d/bokeh_doc'%(host,bok_port))
		self.io_loop.start()

srv = LevServer()
srv.start_server()